#!/usr/bin/env python
# coding: utf-8

# In[19]:


import tkinter as tk

def make_field(parent, side, label_text, label_width, entry_text, entry_width):
    frame = tk.Frame(parent)
    frame.pack(side=side)

    label = tk.Label(frame, width=label_width, text=label_text)
    label.pack(side=tk.LEFT)

    string_var = tk.StringVar(value=entry_text)
    entry = tk.Entry(frame, textvariable=string_var, width=entry_width, justify=tk.RIGHT)
    entry.pack(side=tk.LEFT)

    return string_var


# In[20]:


def find_all_walks(width, height, start_at=(0, 0), end_at=None):
    '''Generate all self-avoiding walks.'''
    walks = []

    # Make a list to show where we have been.
    visited = [[False for h in range(height + 1)] for w in range(width + 1)]

    # Get the number of points we need to visit.
    num_points = (width + 1) * (height + 1)

    # Start the walk at start_at.
    current_walk = []
    current_walk.append(start_at)
    start_x = start_at[0]
    start_y = start_at[1]
    visited[start_x][start_y] = True

    # Search for walks.
    find_walks(end_at, num_points, walks, current_walk,
        start_x, start_y, width, height, visited)
    return walks

def find_walks(end_at, num_points, walks, current_walk, current_x, current_y, width, height, visited):
    '''Extend the walk that is at (current_x, current_y).'''
    # If we have visited every position,
    # then this is a complete walk.
    if len(current_walk) == num_points:
        # Accept if end_at is None or the walk ends at end_at.
        if (end_at is None) or (current_walk[-1] == end_at):
            walks.append(current_walk.copy())
            if len(walks) % 1000 == 0:
                print(f'{len(walks)}')
    else:
        # Try the possible moves.
        next_points = [
            (current_x - 1, current_y),
            (current_x + 1, current_y),
            (current_x, current_y - 1),
            (current_x, current_y + 1),
        ]
        for point in next_points:
            if point[0] < 0: continue
            if point[0] > width: continue
            if point[1] < 0: continue
            if point[1] > height: continue
            if visited[point[0]][point[1]]: continue

            # Try visiting this point.
            visited[point[0]][point[1]] = True
            current_walk.append(point)

            find_walks(end_at, num_points, walks, current_walk,
                point[0], point[1], width, height, visited)

            # We're done visiting this point.
            visited[point[0]][point[1]] = False
            current_walk.pop()


# In[21]:


import tkinter as tk

def draw_circle(canvas, point, radius, color):
    '''Draw a circle centered at this point.'''
    x0 = point[0] - radius
    y0 = point[1] - radius
    x1 = point[0] + radius
    y1 = point[1] + radius
    canvas.create_oval(x0, y0, x1, y1, fill=color)


# In[42]:


import tkinter as tk
import time

# Layout constants.
MARGIN = 5
WINDOW_WID = 500
WINDOW_HGT = 300

class RandomWalksApp:
    # Create and manage the tkinter interface.
    def __init__(self):
        # We are not currently displaying a walk.
        self.walks = None
        self.next_walk_number = -1

        # Make the main interface.
        self.window = tk.Tk()
        self.window.title('self_avoiding_walks_start_end')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry(f'{WINDOW_WID}x{WINDOW_HGT}')

        left_frame = tk.Frame(self.window, width=200)
        left_frame.pack(side=tk.LEFT, fill=tk.Y, expand=False)
        left_frame.pack_propagate(False)

        # Width and height entry fields.
        top_frame =  tk.Frame(left_frame)
        top_frame.pack(side=tk.TOP)

        label_width = 7
        entry_width = 5
        self.width_var = make_field(top_frame, tk.LEFT,
            'Width:', label_width, '4', entry_width)

        spacer = tk.Label(top_frame, text='')
        spacer.pack(side=tk.LEFT, padx=5)

        self.height_var = make_field(top_frame, tk.LEFT,
            'Height:', label_width, '4', entry_width)

        # Start point fields.
        top_frame =  tk.Frame(left_frame)
        top_frame.pack(side=tk.TOP)

        self.start_at_var = make_field(top_frame, tk.LEFT,
            'Start At:', label_width, '0, 0', entry_width)

        spacer = tk.Label(top_frame, text='')
        spacer.pack(side=tk.LEFT, padx=5)

        self.end_at_var = make_field(top_frame, tk.LEFT,
            'End At:', label_width, '4, 4', entry_width)

        # Generate button.
        self.generate_button = tk.Button(left_frame, text='Generate', width=10, command=self.generate)
        self.generate_button.pack(side=tk.TOP, pady=5)
        self.window.bind("<Return>", lambda x: self.generate())

        # Result label.
        self.result_label = tk.Label(left_frame, text=' ')
        self.result_label.pack(side=tk.TOP)

        # Speed scale.
        delay_label = tk.Label(left_frame, text='Delay (ms)):')
        delay_label.pack(side=tk.TOP, anchor=tk.W)
        self.pause_var = tk.IntVar(value=250)
        scale = tk.Scale(left_frame, from_=0, to=1000,
            variable=self.pause_var, orient=tk.HORIZONTAL)
        scale.pack(side=tk.TOP, fill=tk.X)

        # Walk number label.
        self.walk_number_label = tk.Label(left_frame, text=' ')
        self.walk_number_label.pack(side=tk.TOP)

        # Canvas.
        self.canvas = tk.Canvas(self.window, bg='white', cursor='cross',
            borderwidth=2, relief=tk.SUNKEN)
        self.canvas.pack(padx=10, pady=(0, 10),
            side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def generate(self):
        # Clear any previous drawing.
        self.canvas.delete(tk.ALL)
        self.result_label['text'] = 'Working...'

        # Disable the Generate button.
        self.generate_button['state'] = 'disabled'

        # Get parameters.
        self.walk_wid = int(self.width_var.get())
        self.walk_hgt = int(self.height_var.get())

        start_at_text = self.start_at_var.get()
        self.start_at = (
            int(start_at_text.split(',')[0]),
            int(start_at_text.split(',')[1]))

        end_at_text = self.end_at_var.get()
        if end_at_text == '':
            self.end_at = None
        else:
            self.end_at = (
                int(end_at_text.split(',')[0]),
                int(end_at_text.split(',')[1]))

        # Find the walks.
        self.walks = find_all_walks(self.walk_wid, self.walk_hgt,
            self.start_at, self.end_at)
        self.next_walk_number = 0 # Display this one first.

        noun = 'walk' if (len(self.walks) == 1) else 'walks'
        self.result_label['text'] = f'Found {len(self.walks)} {noun}'

        # Calculate drawing parameters.
        canvas_wid = self.canvas.winfo_width()
        canvas_hgt = self.canvas.winfo_height()
        MARGIN = 20
        self.dx = (canvas_wid - 2 * MARGIN) / self.walk_wid
        self.dy = (canvas_hgt - 2 * MARGIN) / self.walk_hgt
        if self.dx > self.dy:
            self.dx = self.dy
        else:
            self.dy = self.dx
        self.ox = (canvas_wid - self.walk_wid * self.dx) / 2
        self.oy = (canvas_hgt - self.walk_hgt * self.dy) / 2

        # Display the walks.
        self.display_next_walk()

    def display_next_walk(self):
        # If we have displayed all of the walks, do nothing.
        if self.next_walk_number >= len(self.walks):
            # Enable the Generate button.
            self.generate_button['state'] = 'active'
            return

        # Display the walk number.
        self.walk_number_label['text'] = f'Walk {self.next_walk_number}'

        # Clear any previous drawing.
        self.canvas.delete(tk.ALL)

        # Draw the walk.
        walk = self.walks[self.next_walk_number]
        self.draw_walk(walk)

        # Draw the dots.
        self.draw_dots(walk)

        # Get ready to draw the next one.
        self.next_walk_number += 1
        self.window.after(self.pause_var.get(), self.display_next_walk)

    def draw_dots(self, walk):
        radius = 5
        for x in range(self.walk_wid + 1):
            for y in range(self.walk_hgt + 1):
                point = (
                    self.ox + x * self.dx,
                    self.oy + y * self.dy
                )
                draw_circle(self.canvas, point, radius, 'lightgreen')

        # Draw the start point in a different color.
        x = self.ox + walk[0][0] * self.dx
        y = self.oy + walk[0][1] * self.dy
        draw_circle(self.canvas, (x, y), radius, 'red')

        # Draw the end point in a different color.
        x = self.ox + walk[-1][0] * self.dx
        y = self.oy + walk[-1][1] * self.dy
        draw_circle(self.canvas, (x, y), radius, 'blue')

    def draw_walk(self, walk):
        # print(walk)
        COLOR = 'black'
        points = []
        for point in walk:
            x = self.ox + point[0] * self.dx
            y = self.oy + point[1] * self.dy
            points.append((x, y))
        self.canvas.create_line(points, fill=COLOR)

    def kill_callback(self):
        self.window.destroy()


# In[43]:


app = RandomWalksApp()
print('Done')


# In[ ]:




